'use strict';

import Alpine from 'alpinejs';
import api from './api';
import { listResources } from './helpers';
import { toast } from '../base/toast';
import { CustomFormData } from '../formdata';

export function settingsView() {
    Alpine.data('settings', (path) => ({
        required: [],
        isProcessing: false,
        plans: [],
        plansFetched: false,

        init() {
            listResources('/plans')
                .then(plans => {
                    this.plans = plans;
                    this.plansFetched = true;
                });
        },

        submit() {
            if (this.isProcessing) {
                return;
            }

            this.isProcessing = true;

            let data = new CustomFormData(this.$refs.form);

            api.post(`/options${this.$refs.form.dataset.path || ''}`, data)
                .then(response => {
                    this.isProcessing = false;

                    toast.show(
                        'Changes saved successfully!',
                        'ti ti-square-rounded-check-filled'
                    );
                })
                .catch(error => this.isProcessing = false);
        },

        clearCache() {
            if (this.isProcessing) {
                return;
            }

            this.isProcessing = true;

            api.delete(`/cache`)
                .then(() => {
                    this.isProcessing = false;

                    toast.show(
                        'Cache cleared successfully!',
                        'ti ti-square-rounded-check-filled'
                    );
                })
                .catch(error => this.isProcessing = false);
        }
    }));

    Alpine.data('llm', (models = []) => ({
        models: models,

        setModelName(value, nameRef) {
            let modelString = value.includes('/') ? value.split('/').slice(1).join('/') : value;
            modelString = modelString.split(':')[0];
            modelString = modelString.replace(/-/g, ' ');
            modelString = modelString.replace(/_/g, '.');
            modelString = modelString.replace(/(\d+(?:\.\d+)?)/g, ' $1 ');
            modelString = modelString
                .split('/')
                .map(part =>
                    part
                        .trim()
                        .split(' ')
                        .filter(word => word)
                        .map(word =>
                            word.charAt(0).toUpperCase() +
                            (word.slice(1).match(/[A-Z]/) ? word.slice(1) : word.slice(1).toLowerCase())
                        )
                        .join(' ')
                )
                .join('/');

            nameRef.value = modelString;
        },

        init() {
            this.models.forEach(model => {
                model.tools = model.tools === '1' || model.tools === true;
                model.vision = model.vision === '1' || model.vision === true;
            });

            if (this.models.length === 0) {
                this.models = [
                    {
                        provider: '',
                        key: '',
                        name: '',
                        tools: false,
                        vision: false
                    }
                ];
            }
        },

        addModel() {
            this.models.push({ key: '', name: '' });
        },

        removeModel(index) {
            this.models.splice(index, 1);
        }
    }))

    Alpine.data('colorSchemes', (light, dark, def) => ({
        light: light,
        dark: dark,
        def: def,

        init() {
            ['light', 'dark'].forEach((scheme) => {
                this.$watch(scheme, (val) => {
                    if (!val) {
                        scheme == 'light' ? this.dark = true : this.light = true;

                        if (this.def == scheme) {
                            this.def = 'system';
                        }
                    }
                });
            });
        },
    }))
}